=== modified file 'bzrlib/_groupcompress_pyx.pyx'
--- bzrlib/_groupcompress_pyx.pyx	2011-05-12 11:38:39 +0000
+++ bzrlib/_groupcompress_pyx.pyx	2011-05-14 10:28:59 +0000
@@ -74,7 +74,7 @@
     int get_hash_offset(delta_index *index, int pos, unsigned int *hash_offset)
     int get_entry_summary(delta_index *index, int pos,
                           unsigned int *global_offset, unsigned int *hash_val)
-    unsigned int c_rabin_hash "rabin_hash" (char *data)
+    unsigned int rabin_hash (char *data)
 
 
 cdef void *safe_malloc(size_t count) except NULL:
@@ -118,13 +118,13 @@
     return AssertionError("Unrecognised delta result code: %d" % result)
 
 
-def rabin_hash(content):
+def _rabin_hash(content):
     if not PyString_CheckExact(content):
         raise ValueError('content must be a string')
     if len(content) < 16:
         raise ValueError('content must be at least 16 bytes long')
     # Try to cast it to an int, if it can fit
-    return int(c_rabin_hash(PyString_AS_STRING(content)))
+    return int(rabin_hash(PyString_AS_STRING(content)))
 
 
 cdef class DeltaIndex:
@@ -137,18 +137,18 @@
     cdef delta_index *_index
     cdef public unsigned long _source_offset
     cdef readonly unsigned int _max_num_sources
-    cdef public int _max_entries_per_source
+    cdef public int _max_bytes_to_index
 
-    def __init__(self, source=None, max_entries_per_source=None):
+    def __init__(self, source=None, max_bytes_to_index=None):
         self._sources = []
         self._index = NULL
         self._max_num_sources = 65000
         self._source_infos = <source_info *>safe_malloc(sizeof(source_info)
                                                         * self._max_num_sources)
         self._source_offset = 0
-        self._max_entries_per_source = 0
-        if max_entries_per_source is not None:
-            self._max_entries_per_source = max_entries_per_source
+        self._max_bytes_to_index = 0
+        if max_bytes_to_index is not None:
+            self._max_bytes_to_index = max_bytes_to_index
 
         if source is not None:
             self.add_source(source, 0)
@@ -299,7 +299,7 @@
         if source_location != 0:
             with nogil:
                 res = create_delta_index(src, self._index, &index,
-                                         self._max_entries_per_source)
+                                         self._max_bytes_to_index)
             if res != DELTA_OK:
                 raise _translate_delta_failure(res)
             if index != self._index:
@@ -317,7 +317,7 @@
         # will always create a new index unless there's a malloc failure
         with nogil:
             res = create_delta_index(&self._source_infos[0], NULL, &index,
-                                     self._max_entries_per_source)
+                                     self._max_bytes_to_index)
         if res != DELTA_OK:
             raise _translate_delta_failure(res)
         self._index = index

=== modified file 'bzrlib/delta.h'
--- bzrlib/delta.h	2011-05-12 11:38:39 +0000
+++ bzrlib/delta.h	2011-05-14 10:28:59 +0000
@@ -42,15 +42,16 @@
  * The given buffer must not be freed nor altered before free_delta_index() is
  * called. The resultant struct must be freed using free_delta_index().
  *
- * :param max_entries: Limit the number of regions to sample to this amount.
- *      Useful if src can be unbounded in size, but you are willing to trade
- *      match accuracy for peak memory.
+ * :param max_bytes_to_index: Limit the number of regions to sample to this
+ *      amount of text. We will store at most max_bytes_to_index / RABIN_WINDOW
+ *      pointers into the source text.  Useful if src can be unbounded in size,
+ *      and you are willing to trade match accuracy for peak memory.
  */
 extern delta_result
 create_delta_index(const struct source_info *src,
                    struct delta_index *old,
                    struct delta_index **fresh,
-                   int max_entries);
+                   int max_bytes_to_index);
 
 
 /*

=== modified file 'bzrlib/diff-delta.c'
--- bzrlib/diff-delta.c	2011-05-12 11:38:39 +0000
+++ bzrlib/diff-delta.c	2011-05-14 10:28:59 +0000
@@ -377,10 +377,10 @@
 create_delta_index(const struct source_info *src,
                    struct delta_index *old,
                    struct delta_index **fresh,
-                   int max_entries)
+                   int max_bytes_to_index)
 {
     unsigned int i, hsize, hmask, num_entries, prev_val, *hash_count;
-    unsigned int total_num_entries, stride;
+    unsigned int total_num_entries, stride, max_entries;
     const unsigned char *data, *buffer;
     struct delta_index *index;
     struct unpacked_index_entry *entry, **hash;
@@ -396,12 +396,15 @@
      */
     stride = RABIN_WINDOW;
     num_entries = (src->size - 1)  / RABIN_WINDOW;
-    if (max_entries > 0 && num_entries > max_entries) {
-        /* Limit the max number of matching entries. This reduces the 'best'
-         * possible match, but means we don't consume all of ram.
-         */
-        num_entries = max_entries;
-        stride = (src->size - 1) / num_entries;
+    if (max_bytes_to_index > 0) {
+        max_entries = max_bytes_to_index / RABIN_WINDOW;
+        if (num_entries > max_entries) {
+            /* Limit the max number of matching entries. This reduces the 'best'
+             * possible match, but means we don't consume all of ram.
+             */
+            num_entries = max_entries;
+            stride = (src->size - 1) / num_entries;
+        }
     }
     if (old != NULL)
         total_num_entries = num_entries + old->num_entries;

=== modified file 'bzrlib/groupcompress.py'
--- bzrlib/groupcompress.py	2011-05-12 15:09:51 +0000
+++ bzrlib/groupcompress.py	2011-05-14 10:28:59 +0000
@@ -491,25 +491,25 @@
     _full_enough_block_size = 3*1024*1024 # size at which we won't repack
     _full_enough_mixed_block_size = 2*768*1024 # 1.5MB
 
-    def __init__(self, block, get_max_entries_per_source=None):
+    def __init__(self, block, get_compressor_settings=None):
         self._block = block
         # We need to preserve the ordering
         self._factories = []
         self._last_byte = 0
-        self._get_max = get_max_entries_per_source
-        self._max_entries_per_source = None
+        self._get_settings = get_compressor_settings
+        self._compressor_settings = None
 
-    def _get_max_entries_per_source(self):
-        if self._max_entries_per_source is not None:
-            return self._max_entries_per_source
-        max_entries = None
-        if self._get_max is not None:
-            max_entries = self._get_max()
-        if max_entries is None:
+    def _get_compressor_settings(self):
+        if self._compressor_settings is not None:
+            return self._compressor_settings
+        settings = None
+        if self._get_settings is not None:
+            settings = self._get_settings()
+        if settings is None:
             vf = GroupCompressVersionedFiles
-            max_entries = vf._DEFAULT_MAX_ENTRIES_PER_SOURCE
-        self._max_entries_per_source = max_entries
-        return self._max_entries_per_source
+            settings = vf._DEFAULT_COMPRESSOR_SETTINGS
+        self._compressor_settings = settings
+        return self._compressor_settings
 
     def add_factory(self, key, parents, start, end):
         if not self._factories:
@@ -549,7 +549,7 @@
         self._block = new_block
 
     def _make_group_compressor(self):
-        return GroupCompressor(self._get_max_entries_per_source())
+        return GroupCompressor(self._get_compressor_settings())
 
     def _rebuild_block(self):
         """Create a new GroupCompressBlock with only the referenced texts."""
@@ -574,7 +574,7 @@
         # TODO: If the content would have expanded, then we would want to
         #       handle a case where we need to split the block.
         #       Now that we have a user-tweakable option
-        #       (max_entries_per_source), it is possible that one person set it
+        #       (max_bytes_to_index), it is possible that one person set it
         #       to a very low value, causing poor compression.
         delta = time.time() - tstart
         self._block = new_block
@@ -933,7 +933,7 @@
 
 class PythonGroupCompressor(_CommonGroupCompressor):
 
-    def __init__(self, max_entries_per_source=None):
+    def __init__(self, max_bytes_to_index=None):
         """Create a GroupCompressor.
 
         Used only if the pyrex version is not available.
@@ -992,10 +992,14 @@
        left side.
     """
 
-    def __init__(self, max_entries_per_source=None):
+    def __init__(self, settings=None):
         super(PyrexGroupCompressor, self).__init__()
-        self._delta_index = DeltaIndex(
-            max_entries_per_source=max_entries_per_source)
+        if settings is None:
+            max_bytes_to_index = \
+                GroupCompressVersionedFiles._DEFAULT_MAX_BYTES_TO_INDEX
+        else:
+            (max_bytes_to_index,) = settings
+        self._delta_index = DeltaIndex(max_bytes_to_index=max_bytes_to_index)
 
     def _compress(self, key, bytes, max_delta_size, soft=False):
         """see _CommonGroupCompressor._compress"""
@@ -1097,7 +1101,7 @@
         currently pending batch.
     """
 
-    def __init__(self, gcvf, locations, get_max_entries_per_source=None):
+    def __init__(self, gcvf, locations, get_compressor_settings=None):
         self.gcvf = gcvf
         self.locations = locations
         self.keys = []
@@ -1106,7 +1110,7 @@
         self.total_bytes = 0
         self.last_read_memo = None
         self.manager = None
-        self._get_max_entries_per_source = get_max_entries_per_source
+        self._get_compressor_settings = get_compressor_settings
 
     def add_key(self, key):
         """Add another to key to fetch.
@@ -1187,7 +1191,7 @@
                 else:
                     block = self.batch_memos[read_memo]
                 self.manager = _LazyGroupContentManager(block,
-                    get_max_entries_per_source=self._get_max_entries_per_source)
+                    get_compressor_settings=self._get_compressor_settings)
                 self.last_read_memo = read_memo
             start, end = index_memo[3:5]
             self.manager.add_factory(key, parents, start, end)
@@ -1211,7 +1215,8 @@
     # local blocks. Either way, 'improved resolution' is not very helpful,
     # versus running out of memory trying to track everything. The default max
     # gives 100% sampling of a 1MB file.
-    _DEFAULT_MAX_ENTRIES_PER_SOURCE = 1024 * 1024 / 16
+    _DEFAULT_MAX_BYTES_TO_INDEX = 1024 * 1024
+    _DEFAULT_COMPRESSOR_SETTINGS = (_DEFAULT_MAX_BYTES_TO_INDEX,)
 
     def __init__(self, index, access, delta=True, _unadded_refs=None,
                  _group_cache=None):
@@ -1233,7 +1238,7 @@
             _group_cache = LRUSizeCache(max_size=50*1024*1024)
         self._group_cache = _group_cache
         self._immediate_fallback_vfs = []
-        self._max_entries_per_source = None
+        self._max_bytes_to_index = None
 
     def without_fallbacks(self):
         """Return a clone of this object without any fallbacks configured."""
@@ -1614,7 +1619,7 @@
         #  - we run out of keys, or
         #  - the total bytes to retrieve for this batch > BATCH_SIZE
         batcher = _BatchingBlockFetcher(self, locations,
-            get_max_entries_per_source=self._get_max_entries_per_source)
+            get_compressor_settings=self._get_compressor_settings)
         for source, keys in source_keys:
             if source is self:
                 for key in keys:
@@ -1666,29 +1671,29 @@
         for _ in self._insert_record_stream(stream, random_id=False):
             pass
 
-    def _get_max_entries_per_source(self):
-        if self._max_entries_per_source is None:
+    def _get_compressor_settings(self):
+        if self._max_bytes_to_index is None:
             # TODO: VersionedFiles don't know about their containing
             #       repository, so they don't have much of an idea about their
             #       location. So for now, this is only a global option.
             c = config.GlobalConfig()
-            val = c.get_user_option('bzr.groupcompress.max_entries_per_source')
+            val = c.get_user_option('bzr.groupcompress.max_bytes_to_index')
             if val is not None:
                 try:
                     val = int(val)
                 except ValueError, e:
                     trace.warning('Value for '
-                                  '"bzr.groupcompress.max_entries_per_source"'
+                                  '"bzr.groupcompress.max_bytes_to_index"'
                                   ' %r is not an integer'
                                   % (val,))
                     val = None
             if val is None:
-                val = self._DEFAULT_MAX_ENTRIES_PER_SOURCE
-            self._max_entries_per_source = val
-        return self._max_entries_per_source
+                val = self._DEFAULT_MAX_BYTES_TO_INDEX
+            self._max_bytes_to_index = val
+        return (self._max_bytes_to_index,)
 
     def _make_group_compressor(self):
-        return GroupCompressor(self._get_max_entries_per_source())
+        return GroupCompressor(self._get_compressor_settings())
 
     def _insert_record_stream(self, stream, random_id=False, nostore_sha=None,
                               reuse_blocks=True):

=== modified file 'bzrlib/tests/test__groupcompress.py'
--- bzrlib/tests/test__groupcompress.py	2011-05-12 11:38:39 +0000
+++ bzrlib/tests/test__groupcompress.py	2011-05-14 10:28:59 +0000
@@ -282,7 +282,7 @@
                         for idx, (text_offset, hash_val)
                          in enumerate(entry_list)
                          if text_offset != 0 or hash_val != 0]
-        rabin_hash = self._gc_module.rabin_hash
+        rabin_hash = self._gc_module._rabin_hash
         self.assertEqual([(8, 16, rabin_hash(_text1[1:17])),
                           (25, 48, rabin_hash(_text1[33:49])),
                           (34, 32, rabin_hash(_text1[17:33])),
@@ -305,7 +305,7 @@
                         for idx, (text_offset, hash_val)
                          in enumerate(entry_list)
                          if text_offset != 0 or hash_val != 0]
-        rabin_hash = self._gc_module.rabin_hash
+        rabin_hash = self._gc_module._rabin_hash
         self.assertEqual([(8, 16, rabin_hash(_text1[1:17])),
                           (9, start2+16, rabin_hash(_text2[1:17])),
                           (25, 48, rabin_hash(_text1[33:49])),
@@ -332,9 +332,9 @@
         self.assertTrue(di._has_index())
         self.assertEqual('N\x90/\x1fdiffer from\nagainst other text\n', delta)
 
-    def test_add_source_max_entries(self):
+    def test_add_source_max_bytes_to_index(self):
         di = self._gc_module.DeltaIndex()
-        di._max_entries_per_source = 3
+        di._max_bytes_to_index = 3*16
         di.add_source(_text1, 0) # (77 bytes -1) // 3 = 25 byte stride
         di.add_source(_text3, 3) # (135 bytes -1) // 3 = 44 byte stride
         start2 = len(_text1) + 3
@@ -344,7 +344,7 @@
         just_entries = sorted([(text_offset, hash_val)
                                for text_offset, hash_val in entry_list
                                 if text_offset != 0 or hash_val != 0])
-        rabin_hash = self._gc_module.rabin_hash
+        rabin_hash = self._gc_module._rabin_hash
         self.assertEqual([(25, rabin_hash(_text1[10:26])),
                           (50, rabin_hash(_text1[35:51])),
                           (75, rabin_hash(_text1[60:76])),

=== modified file 'bzrlib/tests/test_groupcompress.py'
--- bzrlib/tests/test_groupcompress.py	2011-05-13 17:17:30 +0000
+++ bzrlib/tests/test_groupcompress.py	2011-05-14 10:28:59 +0000
@@ -553,21 +553,21 @@
                     'as-requested', False)]
         self.assertEqual([('b',), ('a',), ('d',), ('c',)], keys)
 
-    def test_get_record_stream_max_entries_per_source_default(self):
+    def test_get_record_stream_max_bytes_to_index_default(self):
         vf = self.make_test_vf(True, dir='source')
         vf.add_lines(('a',), (), ['lines\n'])
         vf.writer.end()
         record = vf.get_record_stream([('a',)], 'unordered', True).next()
-        self.assertEqual(vf._DEFAULT_MAX_ENTRIES_PER_SOURCE,
-                         record._manager._get_max_entries_per_source())
+        self.assertEqual(vf._DEFAULT_COMPRESSOR_SETTINGS,
+                         record._manager._get_compressor_settings())
 
-    def test_get_record_stream_accesses_max_entries_per_source_default(self):
+    def test_get_record_stream_accesses_compressor_settings(self):
         vf = self.make_test_vf(True, dir='source')
         vf.add_lines(('a',), (), ['lines\n'])
         vf.writer.end()
-        vf._max_entries_per_source = 1234
+        vf._max_bytes_to_index = 1234
         record = vf.get_record_stream([('a',)], 'unordered', True).next()
-        self.assertEqual(1234, record._manager._get_max_entries_per_source())
+        self.assertEqual((1234,), record._manager._get_compressor_settings())
 
     def test_insert_record_stream_reuses_blocks(self):
         vf = self.make_test_vf(True, dir='source')
@@ -798,36 +798,36 @@
         self.addCleanup(groupcompress.cleanup_pack_group, vf)
         return vf
 
-    def test_max_entries_per_source_default(self):
-        vf = self.make_test_vf()
-        gc = vf._make_group_compressor()
-        self.assertEqual(vf._DEFAULT_MAX_ENTRIES_PER_SOURCE,
-                         vf._max_entries_per_source)
-        if isinstance(gc, groupcompress.PyrexGroupCompressor):
-            self.assertEqual(vf._DEFAULT_MAX_ENTRIES_PER_SOURCE,
-                             gc._delta_index._max_entries_per_source)
-
-    def test_max_entries_per_source_in_config(self):
-        c = config.GlobalConfig()
-        c.set_user_option('bzr.groupcompress.max_entries_per_source', '10000')
-        vf = self.make_test_vf()
-        gc = vf._make_group_compressor()
-        self.assertEqual(10000, vf._max_entries_per_source)
-        if isinstance(gc, groupcompress.PyrexGroupCompressor):
-            self.assertEqual(10000, gc._delta_index._max_entries_per_source)
-
-    def test_max_entries_per_source_bad_config(self):
-        c = config.GlobalConfig()
-        c.set_user_option('bzr.groupcompress.max_entries_per_source', 'boogah')
+    def test_max_bytes_to_index_default(self):
+        vf = self.make_test_vf()
+        gc = vf._make_group_compressor()
+        self.assertEqual(vf._DEFAULT_MAX_BYTES_TO_INDEX,
+                         vf._max_bytes_to_index)
+        if isinstance(gc, groupcompress.PyrexGroupCompressor):
+            self.assertEqual(vf._DEFAULT_MAX_BYTES_TO_INDEX,
+                             gc._delta_index._max_bytes_to_index)
+
+    def test_max_bytes_to_index_in_config(self):
+        c = config.GlobalConfig()
+        c.set_user_option('bzr.groupcompress.max_bytes_to_index', '10000')
+        vf = self.make_test_vf()
+        gc = vf._make_group_compressor()
+        self.assertEqual(10000, vf._max_bytes_to_index)
+        if isinstance(gc, groupcompress.PyrexGroupCompressor):
+            self.assertEqual(10000, gc._delta_index._max_bytes_to_index)
+
+    def test_max_bytes_to_index_bad_config(self):
+        c = config.GlobalConfig()
+        c.set_user_option('bzr.groupcompress.max_bytes_to_index', 'boogah')
         vf = self.make_test_vf()
         # TODO: This is triggering a warning, we might want to trap and make
         #       sure it is readable.
         gc = vf._make_group_compressor()
-        self.assertEqual(vf._DEFAULT_MAX_ENTRIES_PER_SOURCE,
-                         vf._max_entries_per_source)
+        self.assertEqual(vf._DEFAULT_MAX_BYTES_TO_INDEX,
+                         vf._max_bytes_to_index)
         if isinstance(gc, groupcompress.PyrexGroupCompressor):
-            self.assertEqual(vf._DEFAULT_MAX_ENTRIES_PER_SOURCE,
-                             gc._delta_index._max_entries_per_source)
+            self.assertEqual(vf._DEFAULT_MAX_BYTES_TO_INDEX,
+                             gc._delta_index._max_bytes_to_index)
 
 
 class StubGCVF(object):
@@ -1105,39 +1105,42 @@
             self.assertEqual(self._texts[record.key],
                              record.get_bytes_as('fulltext'))
 
-    def test_manager_default_max_entries_per_source(self):
+    def test_manager_default_compressor_settings(self):
         locations, old_block = self.make_block(self._texts)
         manager = groupcompress._LazyGroupContentManager(old_block)
         gcvf = groupcompress.GroupCompressVersionedFiles
-        # It doesn't greedily evaluate _max_entries_per_source
-        self.assertIs(None, manager._max_entries_per_source)
-        self.assertEqual(gcvf._DEFAULT_MAX_ENTRIES_PER_SOURCE,
-                         manager._get_max_entries_per_source())
+        # It doesn't greedily evaluate _max_bytes_to_index
+        self.assertIs(None, manager._compressor_settings)
+        self.assertEqual(gcvf._DEFAULT_COMPRESSOR_SETTINGS,
+                         manager._get_compressor_settings())
 
-    def test_manager_custom_max_entries_per_source(self):
+    def test_manager_custom_compressor_settings(self):
         locations, old_block = self.make_block(self._texts)
         called = []
-        def max_entries():
+        def compressor_settings():
             called.append('called')
-            return 10
+            return (10,)
         manager = groupcompress._LazyGroupContentManager(old_block,
-            get_max_entries_per_source=max_entries)
+            get_compressor_settings=compressor_settings)
         gcvf = groupcompress.GroupCompressVersionedFiles
-        # It doesn't greedily evaluate _max_entries_per_source
-        self.assertIs(None, manager._max_entries_per_source)
-        self.assertEqual(10, manager._get_max_entries_per_source())
-        self.assertEqual(10, manager._get_max_entries_per_source())
-        self.assertEqual(10, manager._max_entries_per_source)
+        # It doesn't greedily evaluate compressor_settings
+        self.assertIs(None, manager._compressor_settings)
+        self.assertEqual((10,), manager._get_compressor_settings())
+        self.assertEqual((10,), manager._get_compressor_settings())
+        self.assertEqual((10,), manager._compressor_settings)
         # Only called 1 time
         self.assertEqual(['called'], called)
 
-    def test__rebuild_handles_max_entries_per_source(self):
+    def test__rebuild_handles_compressor_settings(self):
+        if not isinstance(groupcompress.GroupCompressor,
+                          groupcompress.PyrexGroupCompressor):
+            raise tests.TestNotApplicable('pure-python compressor'
+                ' does not handle compressor_settings')
         locations, old_block = self.make_block(self._texts)
         manager = groupcompress._LazyGroupContentManager(old_block,
-            get_max_entries_per_source=lambda: 2)
+            get_compressor_settings=lambda: (32,))
         gc = manager._make_group_compressor()
-        if isinstance(gc, groupcompress.PyrexGroupCompressor):
-            self.assertEqual(2, gc._delta_index._max_entries_per_source)
+        self.assertEqual(32, gc._delta_index._max_bytes_to_index)
         self.add_key_to_manager(('key3',), locations, old_block, manager)
         self.add_key_to_manager(('key4',), locations, old_block, manager)
         action, last_byte, total_bytes = manager._check_rebuild_action()
@@ -1145,7 +1148,7 @@
         manager._rebuild_block()
         new_block = manager._block
         self.assertIsNot(old_block, new_block)
-        # Because of the new max_entries_per_source, we do a poor job of
+        # Because of the new max_bytes_to_index, we do a poor job of
         # rebuilding. This is a side-effect of the change, but at least it does
         # show the setting had an effect.
         self.assertTrue(old_block._content_length < new_block._content_length)

